package com.sun.media.sound;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;

public final class RIFFReader extends InputStream
{
    private final RIFFReader root;
    private long filepointer;
    private final String fourcc;
    private String riff_type;
    private long ckSize;
    private InputStream stream;
    private long avail;
    private RIFFReader lastiterator;
    
    public RIFFReader(InputStream stream) throws IOException {
        this.filepointer = 0L;
        this.riff_type = null;
        this.ckSize = 2147483647L;
        this.avail = 2147483647L;
        this.lastiterator = null;
        if ((stream instanceof RIFFReader)) {
            this.root = ((RIFFReader)stream).root;
        }
        else {
            this.root = this;
        }
        this.stream = stream;
        int i;
        do {
            i = this.read();
            if (i == -1) {
                this.fourcc = "";
                this.riff_type = null;
                this.avail = 0L;
                return;
            }
            continue;
        } while (i == 0);
        byte[] bytes = new byte[4];
        bytes[0] = (byte)i;
        this.readFully(bytes, 1, 3);
        this.fourcc = new String(bytes, "ascii");
        this.ckSize = this.readUnsignedInt();
        this.avail = this.ckSize;
        if (this.getFormat().equals("RIFF") || this.getFormat().equals("LIST")) {
            if (this.avail > 2147483647L) {
                throw new RIFFInvalidDataException("Chunk size too big");
            }
            byte[] bytes2 = new byte[4];
            this.readFully(bytes2);
            this.riff_type = new String(bytes2, "ascii");
        }
    }
    
    public long getFilePointer() throws IOException {
        return this.root.filepointer;
    }
    
    public boolean hasNextChunk() throws IOException {
        if (this.lastiterator != null) {
            this.lastiterator.finish();
        }
        return this.avail != 0L;
    }
    
    public RIFFReader nextChunk() throws IOException {
        if (this.lastiterator != null) {
            this.lastiterator.finish();
        }
        if (this.avail == 0L) {
            return null;
        }
        return this.lastiterator = new RIFFReader(this);
    }
    
    public String getFormat() {
        return this.fourcc;
    }
    
    public String getType() {
        return this.riff_type;
    }
    
    public long getSize() {
        return this.ckSize;
    }
    
    @Override
    public int read() throws IOException {
        if (this.avail == 0L) {
            return -1;
        }
        int read = this.stream.read();
        if (read == -1) {
            this.avail = 0L;
            return -1;
        }
        --this.avail;
        ++this.filepointer;
        return read;
    }
    
    @Override
    public int read(byte[] array, int n, int len) throws IOException {
        if (this.avail == 0L) {
            return -1;
        }
        if ((long)len > this.avail) {
            int read2 = this.stream.read(array, n, (int)this.avail);
            if (read2 != -1) {
                this.filepointer += (long)read2;
            }
            this.avail = 0L;
            return read2;
        }
        else {
            int read = this.stream.read(array, n, len);
            if (read == -1) {
                this.avail = 0L;
                return -1;
            }
            this.avail -= (long)read;
            this.filepointer += (long)read;
            return read;
        }
    }
    
    public final void readFully(byte[] array) throws IOException {
        this.readFully(array, 0, array.length);
    }
    
    public final void readFully(byte[] array, int n, int i) throws IOException {
        if (i < 0) {
            throw new IndexOutOfBoundsException();
        }
        while (i > 0) {
            int read = this.read(array, n, i);
            if (read < 0) {
                throw new EOFException();
            }
            if (read == 0) {
                Thread.yield();
            }
            n += read;
            i -= read;
        }
        return;
    }
    
    @Override
    public long skip(long a) throws IOException {
        if (a <= 0L || this.avail == 0L) {
            return 0L;
        }
        long min;
        long min2;
        for (min = Math.min(a, this.avail); min > 0L; min -= min2, this.avail -= min2, this.filepointer += min2) {
            min2 = Math.min(this.stream.skip(min), min);
            if (min2 == 0L) {
                Thread.yield();
                if (this.stream.read() == -1) {
                    this.avail = 0L;
                    break;
                }
                min2 = 1L;
            }
        }
        return a - min;
    }
    
    @Override
    public int available() {
        return (int)this.avail;
    }
    
    public void finish() throws IOException {
        if (this.avail != 0L) {
            this.skip(this.avail);
        }
    }
    
    public String readString(int n) throws IOException {
        byte[] array;
        try {
            array = new byte[n];
        }
        catch (OutOfMemoryError cause) {
            throw new IOException("Length too big", cause);
        }
        this.readFully(array);
        int i = 0;
        while (i < array.length) {
            if (array[i] == 0) {
                return new String(array, 0, i, "ascii");
            }
            ++i;
        }
        return new String(array, "ascii");
    }
    
    public byte readByte() throws IOException {
        int read = this.read();
        if (read < 0) {
            throw new EOFException();
        }
        return (byte)read;
    }
    
    public short readShort() throws IOException {
        int read = this.read();
        int read2 = this.read();
        if (read < 0) {
            throw new EOFException();
        }
        if (read2 < 0) {
            throw new EOFException();
        }
        return (short)(read | read2 << 8);
    }
    
    public int readInt() throws IOException {
        int read = this.read();
        int read2 = this.read();
        int read3 = this.read();
        int read4 = this.read();
        if (read < 0) {
            throw new EOFException();
        }
        if (read2 < 0) {
            throw new EOFException();
        }
        if (read3 < 0) {
            throw new EOFException();
        }
        if (read4 < 0) {
            throw new EOFException();
        }
        return read + (read2 << 8) | read3 << 16 | read4 << 24;
    }
    
    public long readLong() throws IOException {
        long n = (long)this.read();
        long n2 = (long)this.read();
        long n3 = (long)this.read();
        long n4 = (long)this.read();
        long n5 = (long)this.read();
        long n6 = (long)this.read();
        long n7 = (long)this.read();
        long n8 = (long)this.read();
        if (n < 0L) {
            throw new EOFException();
        }
        if (n2 < 0L) {
            throw new EOFException();
        }
        if (n3 < 0L) {
            throw new EOFException();
        }
        if (n4 < 0L) {
            throw new EOFException();
        }
        if (n5 < 0L) {
            throw new EOFException();
        }
        if (n6 < 0L) {
            throw new EOFException();
        }
        if (n7 < 0L) {
            throw new EOFException();
        }
        if (n8 < 0L) {
            throw new EOFException();
        }
        return n | n2 << 8 | n3 << 16 | n4 << 24 | n5 << 32 | n6 << 40 | n7 << 48 | n8 << 56;
    }
    
    public int readUnsignedByte() throws IOException {
        int read = this.read();
        if (read < 0) {
            throw new EOFException();
        }
        return read;
    }
    
    public int readUnsignedShort() throws IOException {
        int read = this.read();
        int read2 = this.read();
        if (read < 0) {
            throw new EOFException();
        }
        if (read2 < 0) {
            throw new EOFException();
        }
        return read | read2 << 8;
    }
    
    public long readUnsignedInt() throws IOException {
        long n = (long)this.read();
        long n2 = (long)this.read();
        long n3 = (long)this.read();
        long n4 = (long)this.read();
        if (n < 0L) {
            throw new EOFException();
        }
        if (n2 < 0L) {
            throw new EOFException();
        }
        if (n3 < 0L) {
            throw new EOFException();
        }
        if (n4 < 0L) {
            throw new EOFException();
        }
        return n + (n2 << 8) | n3 << 16 | n4 << 24;
    }
    
    @Override
    public void close() throws IOException {
        this.finish();
        if (this == this.root) {
            this.stream.close();
        }
        this.stream = null;
    }
}
